@@ -10,21 +10,25 @@ sourceCpp("src/dsInt.cpp")
 #'
 #' Create a data source with passed data frame.
 #'
-#' @param inDataFrame Name of data frame
+#' @param dataFrame Name of data frame
 #'
 #' @return None
 #' @export
 #'
 #' @examples
 #' # Create a data source and with built in iris data frame.
-#' \donttest{dsCreateWithDataFrame(inDataFrame = iris)}
-dsCreateWithDataFrame <- function (inDataFrame) {
-    columnTypes <- sapply(inDataFrame, typeof)
-    columnNames <- colnames(inDataFrame)
+#' \donttest{dsCreateWithDataFrame(dataFrame = iris)}
+dsCreateWithDataFrame <- function (dataFrame) {
+    if(!is.data.frame(dataFrame)) {
+      message("Passed object is not a data frame\n")
+      return()
+    }
+    columnTypes <- sapply(dataFrame, typeof)
+    columnNames <- colnames(dataFrame)
   
     dsCreate(columnTypes, columnNames)
-    for(i in 1:nrow(inDataFrame)) {
-        values <- sapply(inDataFrame[i,], as.character)
+    for(i in 1:nrow(dataFrame)) {
+        values <- sapply(dataFrame[i,], as.character)
         dsAddValueRow(values);
     }
 }
