@@ -1,20 +1,23 @@
+
 // Copyright 2021 Werner Mueller
 // Released under the GPL (>= 2)
 
 #include <Rcpp.h>
+
 using namespace Rcpp;
 using namespace std;
 
-#include "generativeData.h"
+#include "density.h"
 
 namespace gdInt {
     DataSource* pDataSource = 0;
     GenerativeData* pGenerativeData = 0;
-  
+    
     string inGenerativeDataFileName = "";
     string inDataSourceFileName = "";
     int batchSize = 256;
-    int maxSize = batchSize * 25000;
+    int maxSize = batchSize * 50000;
+    int nNearestNeighbours = 25;
   
     const string cMaxSizeExceeded = "Max size of generative data exceeded";
 }
@@ -150,6 +153,22 @@ void gdGenerativeDataWrite(const std::string& outFileName) {
     }
 }
 
+// [[Rcpp::export]]
+void gdCreateGenerativeData() {
+  try {
+    if(gdInt::pDataSource == 0) {
+      throw string("No datasource");
+    }
+    
+    delete gdInt::pGenerativeData;
+    gdInt::pGenerativeData = new GenerativeData(*gdInt::pDataSource);
+  } catch (const string& e) {
+    ::Rf_error(e.c_str());
+  } catch(...) {
+    ::Rf_error("C++ exception (unknown reason)");
+  }
+}
+
 // [[Rcpp::export]]
 std::vector<float> gdDataSourceGetDataRandom(int rowCount) {
     try {
@@ -200,18 +219,34 @@ std::vector<float> gdGenerativeDataGetNormalizedDataRandom(int rowCount) {
 
 // [[Rcpp::export]]
 std::vector<float> gdGenerativeDataGetDenormalizedDataRandom(int rowCount) {
-  try {
-    if(gdInt::pGenerativeData == 0) {
-      throw string("No generative data");
-    }
+    try {
+        if(gdInt::pGenerativeData == 0) {
+            throw string("No generative data");
+        }
     
-    std::vector<float> v = ((DataSource*)gdInt::pGenerativeData)->getDenormalizedDataRandom(rowCount);
-    return v;
-  } catch (const string& e) {
-    ::Rf_error(e.c_str());
-  } catch(...) {
-    ::Rf_error("C++ exception (unknown reason)");
-  }
+        std::vector<float> v = ((DataSource*)gdInt::pGenerativeData)->getDenormalizedDataRandom(rowCount);
+        return v;
+    } catch (const string& e) {
+        ::Rf_error(e.c_str());
+    } catch(...) {
+        ::Rf_error("C++ exception (unknown reason)");
+    }
+}
+
+// [[Rcpp::export]]
+std::vector<std::vector<float>> gdGenerativeDataGetDenormalizedDataRandomWithDensities(int rowCount) {
+    try {
+        if(gdInt::pGenerativeData == 0) {
+            throw string("No generative data");
+        }
+    
+        std::vector<std::vector<float>> v = gdInt::pGenerativeData->getDenormalizedDataRandomWithDensities(rowCount);
+        return v;
+    } catch (const string& e) {
+        ::Rf_error(e.c_str());
+    } catch(...) {
+        ::Rf_error("C++ exception (unknown reason)");
+    }
 }
 
 // [[Rcpp::export]]
@@ -256,7 +291,6 @@ void gdAddValueRows(const std::vector<float>& valueRows) {
 //' gdGetNumberOfRows()}
 // [[Rcpp::export]]
 int gdGetNumberOfRows() {
-//int gdGetNormalizedSize() {
     try {
         if(gdInt::pGenerativeData == 0) {
             throw string("No gnerative data");
@@ -272,21 +306,21 @@ int gdGetNumberOfRows() {
 
 // [[Rcpp::export]]
 std::vector<std::wstring> gdGetColumnNames(std::vector<int>& indexVector) {
-  try {
-    if(gdInt::pGenerativeData == 0) {
-      throw string("No generative data");
-    }
+    try {
+        if(gdInt::pGenerativeData == 0) {
+            throw string("No generative data");
+        }
     
-    for(int i = 0; i < indexVector.size(); i++) {
-      indexVector[i] -= 1;
+        for(int i = 0; i < indexVector.size(); i++) {
+            indexVector[i] -= 1;
+        }
+        vector<wstring> columnNames = gdInt::pGenerativeData->getColumnNames(indexVector);
+        return columnNames;
+    } catch (const string& e) {
+        ::Rf_error(e.c_str());
+    } catch(...) {
+        ::Rf_error("C++ exception (unknown reason)");
     }
-    vector<wstring> columnNames = gdInt::pGenerativeData->getColumnNames(indexVector);
-    return columnNames;
-  } catch (const string& e) {
-    ::Rf_error(e.c_str());
-  } catch(...) {
-    ::Rf_error("C++ exception (unknown reason)");
-  }
 }
 
 // [[Rcpp::export]]
@@ -307,18 +341,15 @@ int gdGetGenerativeDataDimension() {
 // [[Rcpp::export]]
 std::vector<std::wstring> gdGetNumberVectorIndexNames(std::vector<int>& numberVectorIndices) {
     try {
-        //if(gdInt::pGenerativeDataSource == 0) {
-        if(gdInt::pDataSource == 0) {
-            //throw string("No gnerative datasource");
-            throw string("No datasource");
+        if(gdInt::pGenerativeData == 0) {
+            throw string("No generative data");
         }
     
         vector<int> indexVector = numberVectorIndices;
         for(int i = 0; i < indexVector.size(); i++) {
             indexVector[i] -= 1;
         }
-        //vector<wstring> numberVectorIndexNames = gdInt::pGenerativeDataSource->getNumbeVectorIndexNames(indexVector);
-        vector<wstring> numberVectorIndexNames = gdInt::pDataSource->getNumbeVectorIndexNames(indexVector);
+        vector<wstring> numberVectorIndexNames = gdInt::pGenerativeData->getNumbeVectorIndexNames(indexVector);
         return numberVectorIndexNames;
     } catch (const string& e) {
         ::Rf_error(e.c_str());
@@ -329,64 +360,190 @@ std::vector<std::wstring> gdGetNumberVectorIndexNames(std::vector<int>& numberVe
 
 //' Get a row in generative data
 //'
-//' Get a row in generative data containing values of active columns for a row index
+//' Get a row in generative data for a row index
 //'
 //' @param index Index of row
 //'
-//' @return Row in generative data
+//' @return List containing row in generative data
 //' @export
 //'
 //' @examples
 //' \donttest{gdRead("gd.bin")
-//' gdGetRow(1)}
+//' gdGetRow(1000)}
 // [[Rcpp::export]]
 List gdGetRow(int index) {
     try {
-      if(gdInt::pGenerativeData == 0) {
-        throw string("No generative data");
-    }
-    
-    List list;
-    vector<Column*> const & columnVector = gdInt::pGenerativeData->getColumnVector();
-    for(int j = 0; j < columnVector.size(); j++) {
-        Column::COLUMN_TYPE type = columnVector[j]->getColumnType();
-        if(type == Column::NUMERICAL) {
-          vector<float> numberVector = columnVector[j]->getDenormalizedNumberVector(index - 1);
-          float value = numberVector[0];
-          list.insert(list.end(), value);
-        } else {
-          throw string(cInvalidColumnType);
+        if(gdInt::pGenerativeData == 0) {
+            throw string("No generative data");
         }
-    }
-    return list;
+    
+        List list;
+        vector<Column*> const & columnVector = gdInt::pGenerativeData->getColumnVector();
+        for(int i = 0; i < columnVector.size(); i++) {
+            Column::COLUMN_TYPE type = columnVector[i]->getColumnType();
+            if(type == Column::NUMERICAL) {
+                vector<float> numberVector = columnVector[i]->getDenormalizedNumberVector(index - 1);
+                float value = numberVector[0];
+                list.insert(list.end(), value);
+            } else {
+                throw string(cInvalidColumnType);
+            }
+        }
+        return list;
     } catch (const string& e) {
-      ::Rf_error(e.c_str());
+        ::Rf_error(e.c_str());
     } catch(...) {
-      ::Rf_error("C++ exception (unknown reason)");
+        ::Rf_error("C++ exception (unknown reason)");
     }
 }
 
 // [[Rcpp::export]]
 float gdGetMax(int i) {
+    try {
+        if(gdInt::pGenerativeData == 0) {
+            throw string("No generative data");
+        }
+    
+        if(i - 1 < 0 || i - 1 > gdInt::pGenerativeData->getDimension() - 1) {
+            throw string(cInvalidColumnIndex);
+        }
+        int j = gdInt::pGenerativeData->getColumnIndex(i - 1);
+        vector<Column*> const& columnVector = gdInt::pGenerativeData->getColumnVector();
+    
+        float max = 0;
+        Column::COLUMN_TYPE type = columnVector[j]->getColumnType();
+        if(type == Column::NUMERICAL) {
+            NumberColumn* pNumberColumn = dynamic_cast<NumberColumn*>(columnVector[j]);
+            max = pNumberColumn->getMax();
+        } else {
+            throw string(cInvalidColumnType);
+        }
+        return max;
+    } catch (const string& e) {
+        ::Rf_error(e.c_str());
+    } catch(...) {
+        ::Rf_error("C++ exception (unknown reason)");
+    }
+}
+
+// [[Rcpp::export]]
+float gdGetMin(int i) {
+    try {
+        if(gdInt::pGenerativeData == 0) {
+            throw string("No generative data");
+        }
+      
+        if(i - 1 < 0 || i - 1 > gdInt::pGenerativeData->getDimension() - 1) {
+            throw string(cInvalidColumnIndex);
+        }
+        int j = gdInt::pGenerativeData->getColumnIndex(i - 1);
+        vector<Column*> const& columnVector = gdInt::pGenerativeData->getColumnVector();
+      
+        float min = 0;
+        Column::COLUMN_TYPE type = columnVector[j]->getColumnType();
+        if(type == Column::NUMERICAL) {
+            NumberColumn* pNumberColumn = dynamic_cast<NumberColumn*>(columnVector[j]);
+            min = pNumberColumn->getMin();
+        } else {
+            throw string(cInvalidColumnType);
+        }
+        return min;
+    } catch (const string& e) {
+        ::Rf_error(e.c_str());
+    } catch(...) {
+        ::Rf_error("C++ exception (unknown reason)");
+    }
+}
+
+// [[Rcpp::export]]
+void gdIntCalculateDensityValues() {
+    try {
+        if(gdInt::pGenerativeData == 0) {
+            throw string("No generative data");
+        }
+        
+        VpGenerativeData vpGenerativeData(*gdInt::pGenerativeData);
+        L2Distance l2Distance;
+        Progress progress(gdInt::pGenerativeData->getNormalizedSize());
+        VpTree vpTree(vpGenerativeData, l2Distance, &progress);
+        vpTree.build();
+       
+        Density density(*gdInt::pGenerativeData, vpTree, gdInt::nNearestNeighbours, &progress);
+        density.calculateDensityValues();
+         
+        progress(gdInt::pGenerativeData->getNormalizedSize());
+    } catch (const string& e) {
+        ::Rf_error(e.c_str());
+    } catch(...) {
+        ::Rf_error("C++ exception (unknown reason)");
+    }
+}
+
+//' Calculate density value for a data record
+//' 
+//' Calculate density value for a data record.
+//'
+//' @param dataRecord List containing a data record
+//'
+//' @return Normalized density value number
+//' @export
+//'
+//' @examples
+//' \donttest{gdRead("gd.bin")
+//' gdCalculateDensityValue(List(6.1, 2.6, 5.6, 1.4))}
+// [[Rcpp::export]]
+float gdCalculateDensityValue(List dataRecord) {
   try {
     if(gdInt::pGenerativeData == 0) {
       throw string("No generative data");
     }
     
-    vector<Column*> const & columnVector = gdInt::pGenerativeData->getColumnVector();
-    if(i - 1 < 0 || i - 1 > (int)columnVector.size() - 1) {
-        throw string(cInvalidColumnIndex);
+    vector<float> numberVector;
+    for(List::iterator iterator = dataRecord.begin(); iterator != dataRecord.end(); ++iterator) {
+      float number = (float)as<double>(*iterator);
+      numberVector.push_back(number);
     }
     
-    float max = 0;
-    Column::COLUMN_TYPE type = columnVector[i - 1]->getColumnType();
-    if(type == Column::NUMERICAL) {
-        NumberColumn* pNumberColumn = dynamic_cast<NumberColumn*>(columnVector[i - 1]);
-        max = pNumberColumn->getMax();
-    } else {
-        throw string(cInvalidColumnType);
+    VpGenerativeData vpGenerativeData(*gdInt::pGenerativeData);
+    L2Distance l2Distance;
+    VpTree vpTree(vpGenerativeData, l2Distance, 0);
+    
+    Density density(*gdInt::pGenerativeData, vpTree, gdInt::nNearestNeighbours, 0);
+    float d = density.calculateDensityValue(numberVector);
+    return d;
+  } catch (const string& e) {
+    ::Rf_error(e.c_str());
+  } catch(...) {
+    ::Rf_error("C++ exception (unknown reason)");
+  }
+}
+
+//' Calculate density value quantile
+//' 
+//' Calculate density value quantile for a percent value. 
+//'
+//' @param percent Percent value
+//'
+//' @return Normalized density value quantile number
+//' @export
+//'
+//' @examples
+//' \donttest{gdRead("gd.bin")
+//' gdCalculateDensityValueQuantile(50)}
+// [[Rcpp::export]]
+float gdCalculateDensityValueQuantile(float percent) {
+  try {
+    if(gdInt::pGenerativeData == 0) {
+      throw string("No generative data");
     }
-    return max;
+    
+    VpGenerativeData vpGenerativeData(*gdInt::pGenerativeData);
+    L2Distance l2Distance;
+    VpTree vpTree(vpGenerativeData, l2Distance, 0);
+    
+    Density density(*gdInt::pGenerativeData, vpTree, gdInt::nNearestNeighbours, 0);
+    float q = density.calculateQuantile(percent);
+    return q;
   } catch (const string& e) {
     ::Rf_error(e.c_str());
   } catch(...) {
@@ -395,29 +552,12 @@ float gdGetMax(int i) {
 }
 
 // [[Rcpp::export]]
-float gdGetMin(int i) {
-  try {
-      if(gdInt::pGenerativeData == 0) {
-        throw string("No generative data");
-      }
-    
-      vector<Column*> const & columnVector = gdInt::pGenerativeData->getColumnVector();
-      if(i - 1 < 0 || i - 1 > (int)columnVector.size() - 1) {
-          throw string(cInvalidColumnIndex);
-      }
-    
-      float min = 0;
-      Column::COLUMN_TYPE type = columnVector[i - 1]->getColumnType();
-      if(type == Column::NUMERICAL) {
-          NumberColumn* pNumberColumn = dynamic_cast<NumberColumn*>(columnVector[i - 1]);
-          min = pNumberColumn->getMin();
-      } else {
-          throw string(cInvalidColumnType);
-      }
-      return min;
-  } catch (const string& e) {
-      ::Rf_error(e.c_str());
-  } catch(...) {
-      ::Rf_error("C++ exception (unknown reason)");
-  }
+std::string gdBuildFileName(const std::string& fileName, float niveau) {
+    try {
+        return BuildFileName()(GetFileName()(fileName), niveau, GetExtension()(fileName));
+    } catch (const string& e) {
+        ::Rf_error(e.c_str());
+    } catch(...) {
+        ::Rf_error("C++ exception (unknown reason)");
+    }
 }
